/*
 * extender.h
 *
 *  Created on: 2010/09/28
 *      Author: shu
 */

#ifndef EXTENDER_H_
#define EXTENDER_H_

#include "aligner.h"
#include "encoder.h"
#include "edit_blocks.h"
#include "sequence_type.h"
#include "score_matrix.h"
#include "score_matrix_reader.h"
#include <stdint.h>
#include <vector>

using namespace std;

class AlignerParameters;
class Queries;
class Db;

class Extender {
public:
  static const int kInitScore = -(1 << 10);
  Extender(const AlignerParameters &parameters, Queries &queries);

  virtual ~Extender();

  virtual void GappedExtend(Query &query, Db &db, const Coordinate &hit, int *score, Coordinate *start, Coordinate *end, EditBlocks *edit_blocks);

protected:
  enum EditOpType {
    kSubstitution = 0,
    kGapInQuery = 1,
    kGapInDb = 2,
    kExtendGapInQuery = 4,
    kExtendGapInDb = 8
  };

  static const int kMaxStateArrayLength = (1 << 21);
  static const int kEditOpMask = 3;

  typedef struct {
    int best;
    int best_gap;
  }DpCell;

  typedef struct {
    vector<uint8_t> state_array;
    uint32_t used;
  }GapState;



  void ClearGapState();
  uint32_t GetGapStateIndex(uint32_t length);
  int GappedExtendOneSide(Code *query, Code *db, int *query_offset, int *db_offset, bool reversed, EditBlocks *edit_blocks);
  int GappedExtendOneSideScoreOnly(Code *query, Code *db, int *query_offset, int *db_offset, bool reversed);
  int RestrictedGappedExtendOneSide(Code *query, Code *db, int *query_offset, int *db_offset, bool reversed, EditBlocks *edit_blocks);
  int RestrictedGappedExtendOneSideScoreOnly(Code *query, Code *db, int *query_offset, int *db_offset, bool reversed);

  void PickOptimalPath(const vector<uint8_t*> &edit_script, const vector<uint32_t> &edit_script_offset, int query_position, int db_position, int increment, EditBlocks *edit_blocks);


  uint32_t dp_mem_length_;
  DpCell *dp_mem_;
  int gapped_extend_nominal_cutoff_;
  int final_gapped_extend_nominal_cutoff_;
  int open_gap_;
  int extend_gap_;
  int max_gapped_size_;
  ScoreMatrix score_matrix_;
  vector<int> matrix_;
  uint32_t max_query_length_;
  vector<GapState> gap_state_struct_;
};

#endif /* EXTENDER_H_ */

